//++
//|                                JQS UGA The alternative ZigZag v1.1.0 |
//|                                       Copyright  2010, JQS aka Joo. |
//|                                     http://www.mql4.com/ru/users/joo |
//++
#property copyright "Copyright  2010, JQS aka Joo."                   //|
#property link      "http://www.mql4.com/ru/users/joo"                 //|
//++
#property description "The script that demonstrates the functioning "  //|
#property description "of the Universal Genetic Algorithm UGAlib library"
#property description "that uses the representation of chromosome"     //|
#property description "via real numbers exemplified in the search"     //|
#property description "of peaks of alternative ZZ"                     //|
//++
#property script_show_inputs                                           //|
//++
#include "UGAlib.mqh"
//++

//
//----------------------Input Parameters--------------------------------
input string GenofundParam        =        "----Parameters of gene pool----";
input int    ChromosomeCount_P    = 100;    //Number of chromosomes in colony
input int    GeneCount_P          = 100;    //Number of genes
input int    Epoch_P               = 50;    //Number of epochs without improvement
//---
input string GA_OperatorParam     =        "----Parameters of operators----";
input double ReplicationPortion_P  = 100.0; //Portion of  replication.
input double NMutationPortion_P    = 10.0;  //Portion of natural mutation.
input double ArtificialMutation_P  = 10.0;  //Portion of artificial mutation.
input double GenoMergingPortion_P  = 20.0;  //Portion of adoption of genes.
input double CrossingOverPortion_P = 20.0;  //For crossing over.
input double ReplicationOffset_P   = 0.5;   //Rate of shifting the interval borders
input double NMutationProbability_P= 5.0;   //Probability of mutation of each gene in %
//---
input string OptimisationParam    =        "----Optimization parameters----";
input double RangeMinimum_P       = 0.0;    //Minimum of the search range
input double RangeMaximum_P       = 5.0;    //Maximum of the search range
input double Precision_P          = 1.0;    //Necessary precision
input int    OptimizeMethod_P     = 2;      //Optim.:1-Min,other-Max

input string Other                =        "----Other----";
input double Spred                = 80.0;
input bool   Show                 = true;
//

//
//----------------------Global Variables-----------------------------
double   Hight  [];
double   Low    [];
datetime Time   [];
datetime Ti     [];
double   Peaks  [];
bool     show;
//
//--------------------------Program Body--------------------------------
int OnStart()
{
  //-----------------------Variables-------------------------------------
  //Preparation of global variables for UGA
  ChromosomeCount=ChromosomeCount_P; //Number of chromosomes in colony
  GeneCount      =GeneCount_P;       //Number of genes
  RangeMinimum   =RangeMinimum_P;    //Minimum of the search range
  RangeMaximum   =RangeMaximum_P;    //Maximum of the search range
  Precision      =Precision_P;       //Search step
  OptimizeMethod =OptimizeMethod_P;  //1-minimum, any other-maximum  

  ArrayResize(Chromosome,GeneCount+1);
  ArrayInitialize(Chromosome,0);
  Epoch=Epoch_P;                     //Number of epochs without improvement
  //----------------------------------------------------------------------
  //Preparation of global variables
  ArraySetAsSeries(Hight,true);  CopyHigh (NULL,0,0,GeneCount+1,Hight);
  ArraySetAsSeries(Low,true);    CopyLow  (NULL,0,0,GeneCount+1,Low);
  ArraySetAsSeries(Time,true);   CopyTime (NULL,0,0,GeneCount+1,Time);
  ArrayResize     (Ti,GeneCount+1);ArrayInitialize(Ti,0);
  ArrayResize(Peaks,GeneCount+1);ArrayInitialize(Peaks,0.0);
  show=Show;
  //----------------------------------------------------------------------
  //Local variables
  int time_start=(int)GetTickCount(),time_end=0;
  //----------------------------------------------------------------------

  //Clear screen
  ObjectsDeleteAll(0,-1,-1);
  ChartRedraw(0);
  //Start of main function of UGA
  UGA
  (
  ReplicationPortion_P, //Portion of replication.
  NMutationPortion_P,   //Portion of natural mutation.
  ArtificialMutation_P, //Portion of artificial mutation.
  GenoMergingPortion_P, //Portion of adoption of genes.
  CrossingOverPortion_P,//Portion of crossing over.
  //---
  ReplicationOffset_P,  //Rate of shifting the interval borders
  NMutationProbability_P//Probability of mutation of each gene in %
  );
  //----------------------------------
  //Display the last variant on the screen
  show=true;
  ServiceFunction();
  //----------------------------------
  time_end=(int)GetTickCount();
  //----------------------------------
  Print(time_end-time_start,"  - Time of execution");
  //----------------------------------
  return(0);
}
//

//
//-----------------------------------------------------------------------+
// Service Function. Called from UGA.                                    |
// It is used for displaying the best variant of optimized parameters,   |
// for example, for controlling.                                         |
// If there is no need in it, leave the function empty as following:     |
//   void ServiceFunction()                                              |
//   {                                                                   |
//   }                                                                   |
//-----------------------------------------------------------------------+
void ServiceFunction()
{ 
  if (show==true)
  {
    //-----------------------Variables-----------------------------------
    double PipsSum=0.0;
    int    PeaksCount=0;
    double temp=0.0;
    //--------------------------------------------------------------------
    for (int u=1;u<=GeneCount;u++)
    {
      temp=Chromosome[u];
      if (temp<=1.0 )
      {
        Peaks[PeaksCount]=NormalizeDouble(Hight[u],Digits());
        Ti   [PeaksCount]=Time[u];
        PeaksCount++;
      }
      if (temp>=4.0)
      {
        Peaks[PeaksCount]=NormalizeDouble(Low[u],Digits());
        Ti   [PeaksCount]=Time[u];
        PeaksCount++;
      }
    }
    ObjectsDeleteAll(0,-1,-1);
    for (int V=0;V<PeaksCount-1;V++)
    {
      PipsSum+=(MathAbs(Peaks[V]-Peaks[V+1]))/Point()-Spred;
      ObjectCreate    (0,"BoxBackName"+(string)V,OBJ_TREND,0,Ti[V],Peaks[V],Ti[V+1],Peaks[V+1]);
      ObjectSetInteger(0,"BoxBackName"+(string)V,OBJPROP_COLOR,Black);
      ObjectSetInteger(0,"BoxBackName"+(string)V,OBJPROP_SELECTABLE,true);
    }
    ChartRedraw(0);
    Comment(NormalizeDouble(PipsSum,0));
  }
  //----------------------------------------------------------------------
  else
    return;
}
//

//
//-------------------------------------------------------------------------+
// Function of evaluation the adjustment of an individual. Called from UGA.|
// This is the optimized function itself.                                  |
//                                                                         |
// For example:                                                            |
// We need ti find the optimal parameters of the indicator. We should "run"|
// the indicator on historical data and collect the signals from it. If    |
// we want to know the number of profit points then the maximal value will |
// correspond the maximal fitness of an individual                         |
// (set of parameters of the indicator).                                   |
//-------------------------------------------------------------------------+

//========================================================================
void FitnessFunction(int chromos)
{
  //-----------------------Variables-------------------------------------
  double PipsSum=0.0;
  int    PeaksCount=0;
  double temp=0.0;
  //----------------------------------------------------------------------
  for (int u=1;u<=GeneCount;u++)
  {
    temp=Colony[u][chromos];
    if (temp<=1.0)
    {
      Peaks[PeaksCount]=NormalizeDouble(Hight[u],Digits());
      PeaksCount++;
    }
    if (temp>=4.0)
    {
      Peaks[PeaksCount]=NormalizeDouble(Low[u],Digits());
      PeaksCount++;
    }
  }

  if (PeaksCount>1)
  {
    for (int V=0;V<PeaksCount-1;V++)
      PipsSum+=(MathAbs(Peaks[V]-Peaks[V+1]))/Point()-Spred;

    Colony[0][chromos]=NormalizeDouble(PipsSum,0);
  }
  else
    Colony[0][chromos]=-10000000.0;
  AmountStartsFF++;
}
//